@@ -97,3 +97,9 @@ class MethodNotAllowed(ResponseException):
     def __init__(self, valid_methods):
         super(MethodNotAllowed, self).__init__(valid_methods)
         self.response = Response.method_not_allowed(valid_methods)
+
+
+class MissingRouteArg(BadRequest):
+    """\
+    A request argument was expected but not supplied.
+    """
