@@ -9,8 +9,9 @@ HTTP headers and request data (eg query string or submitted form data).
 import posixpath
 import re
 
-from fresco.compat import string_types, quote, urlparse, urlunparse, \
-                          ParseResult
+from fresco import exceptions
+from fresco.compat import (ustring_type, quote, urlparse, urlunparse,
+                           ParseResult)
 from fresco.util.wsgi import environ_to_unicode
 
 import fresco
@@ -183,9 +184,9 @@ class Request(object):
             ... ''') as c:
             ...     request = c.request
             ...
-            >>> [c.value for c in request.cookies.getlist('Customer')]
+            >>> [item.value for item in request.cookies.getlist('Customer')]
             ['WILE_E_COYOTE']
-            >>> [c.value for c in request.cookies.getlist('Part')]
+            >>> [item.value for item in request.cookies.getlist('Part')]
             ['Rocket_0001', 'Catapult_0032']
 
         See rfc2109, section 4.4
@@ -295,7 +296,7 @@ class Request(object):
             port = environ_to_unicode(self.environ['SERVER_PORT'])
 
         if (scheme == 'http' and port == '80') \
-            or (scheme == 'https' and port == '443'):
+                or (scheme == 'https' and port == '443'):
             netloc = host
         else:
             netloc = host + ':' + port
@@ -316,8 +317,11 @@ class Request(object):
 
         Note that PATH_INFO is already unquoted by the server
         """
-        return environ_to_unicode(self.environ.get('PATH_INFO', ''),
-                                  self.charset)
+        try:
+            return environ_to_unicode(self.environ.get('PATH_INFO', ''),
+                                      self.charset)
+        except UnicodeDecodeError:
+            raise exceptions.BadRequest
 
     @property
     def script_name(self):
@@ -326,8 +330,11 @@ class Request(object):
 
         Note that SCRIPT_NAME is already unquoted by the server
         """
-        return environ_to_unicode(self.environ.get('SCRIPT_NAME', ''),
-                                  self.charset)
+        try:
+            return environ_to_unicode(self.environ.get('SCRIPT_NAME', ''),
+                                      self.charset)
+        except UnicodeDecodeError:
+            raise exceptions.BadRequest
 
     @property
     def query_string(self):
@@ -376,14 +383,9 @@ class Request(object):
         """
         return self.environ.get(self.SESSION_ENV_KEY)
 
-    def make_url(
-        self, scheme=None, netloc=None,
-        path=None, parameters=None,
-        query=None, fragment=None,
-        SCRIPT_NAME=None,
-        PATH_INFO=None,
-        **kwargs
-    ):
+    def make_url(self, scheme=None, netloc=None, path=None, parameters=None,
+                 query=None, query_add=None, query_replace=None, fragment=None,
+                 SCRIPT_NAME=None, PATH_INFO=None, **kwargs):
         r"""\
         Make a new URL based on the current URL, replacing any of the six
         URL elements (scheme, netloc, path, parameters, query or fragment). The
@@ -397,6 +399,12 @@ class Request(object):
                             urlparse module)
         :param fragment:    The URL fragment
         :param query:       Query data, as a list of tuples, a dict or a string
+        :param query_add:   Query parameters to add. If the current url already
+                            contains query parameters with the same name, these
+                            will be supplemented with the new values.
+        :param query_replace: Query parameters to replace. If the current url
+                              already contains query parameters with the same
+                              name, these will be replaced with the new values.
         :param PATH_INFO:   The PATH_INFO portion of the path. Overrides
                             ``path``
         :param SCRIPT_NAME: The SCRIPT_NAME portion of the path. Overrides
@@ -405,9 +413,9 @@ class Request(object):
                             querystring
         :rtype: str
 
-        All arguments (other than ``query``, see examples below) may be
-        byte strings or unicode strings (native strings in python 3). Where
-        necessary values will be converted to utf-8 encoded byte strings.
+        All arguments (other than query related arguments, see examples below)
+        may be byte strings or unicode strings (native strings in python 3).
+        Where necessary values will be converted to utf-8 encoded byte strings.
 
         Examples:
 
@@ -443,6 +451,19 @@ class Request(object):
             request.make_url(query={'portions': '2'}, cherries='n')
             u'http://example.com/fruitsalad/banana?portions=2;cherries='n'
 
+        If a relative path is passed, the returned URL is joined to the old in
+        the same way as a web browser would interpret a relative HREF in a
+        document at the current location::
+
+            >>> request.make_url(path='kiwi')
+            u'http://example.com/fruitsalad/kiwi'
+
+            >>> request.make_url(path='../strawberry')
+            u'http://example.com/strawberry'
+
+            >>> request.make_url(path='../../../plum')
+            u'http://example.com/plum'
+
         The ``query`` argument can take a dictionary, a list of ``(name,
         value)`` pairs or any other type convertable to a MultiDict::
 
@@ -457,19 +478,12 @@ class Request(object):
             ...                         ('b', 'paris')])
             u'http://example.com/fruitsalad/banana?a=tokyo;b=milan;b=paris'
 
-        If a relative path is passed, the returned URL is joined to the old in
-        the same way as a web browser would interpret a relative HREF in a
-        document at the current location::
-
-            >>> request.make_url(path='kiwi')
-            u'http://example.com/fruitsalad/kiwi'
-
-            >>> request.make_url(path='../strawberry')
-            u'http://example.com/strawberry'
-
-            >>> request.make_url(path='../../../plum')
-            u'http://example.com/plum'
-
+        The ``query_add`` and ``query_replace`` arguments add or replace
+        values in an existing query string. If either of these is specified
+        then the current request's query string (or value of ``query``, if
+        specified) will be extended with the given values.
+        These arguments take a dict, a list of ``(key, value)`` tuples, or any
+        other type convertable to a MultiDict.
         """
         url = []
         parsed_url = self.parsed_url
@@ -498,23 +512,36 @@ class Request(object):
             path = parsed_url[2]
 
         if query is not None:
-            if not isinstance(query, string_types):
-                query = make_query(query, **kwargs)
+            if isinstance(query, ustring_type):
+                # Unicode queries should be pre-encoded and thus contain ascii
+                # characters only
+                query = query.encode('ASCII')
             elif isinstance(query, bytes):
                 if kwargs:
                     if query:
                         query += ';'
                     query += make_query(kwargs)
-        else:
-            query = ''
 
-        for specified, parsed in zip((scheme, netloc,
-                                      path, parameters,
-                                      query, fragment), parsed_url):
-            if specified is not None:
-                url.append(specified)
-            else:
-                url.append(parsed)
+        if (query_add or query_replace) and isinstance(query, bytes):
+            query = MultiDict(parse_querystring(query.decode('ASCII')))
+
+        if query_add is not None:
+            query = query if query is not None else self.query.copy()
+            query.extend(query_add)
+
+        if query_replace is not None:
+            query = query if query is not None else self.query.copy()
+            query.update(query_replace)
+
+        if query and not isinstance(query, bytes):
+            query = make_query(query, **kwargs)
+
+        url = [scheme if scheme is not None else parsed_url[0],
+               netloc if netloc is not None else parsed_url[1],
+               path if path is not None else parsed_url[2],
+               parameters if parameters is not None else parsed_url[3],
+               query,
+               fragment]
 
         return urlunparse(url)
 
