@@ -21,9 +21,7 @@ class RequestContext(object):
         self.push()
 
     def push(self, **bindnames):
-        contexts = self._contexts[self._ident_func()]
-        c = dict(contexts[-1] if contexts else {}, **bindnames)
-        self._contexts[self._ident_func()].append(c)
+        self._contexts[self._ident_func()].append(bindnames)
 
     def pop(self):
         ident = self._ident_func()
@@ -58,9 +56,11 @@ class RequestContext(object):
         return self.currentcontext().get(item, default)
 
     def __repr__(self):
-        return "<%s depth=%d; current=%r>" % (self.__class__.__name__,
-                                              len(self._contexts),
-                                              self.currentcontext())
+        return "<%s localdepth=%d; total=%d; current=%r>" % (
+                            self.__class__.__name__,
+                            len(self._contexts[self._ident_func()]),
+                            sum(len(ctx) for ctx in self._contexts.values()),
+                            self.currentcontext())
 
 
 #: The Context for the current request; allows apps to access the current
