@@ -7,7 +7,7 @@ from fresco.compat import string_types
 from fresco.exceptions import ResponseException
 from fresco.response import Response
 from fresco.requestcontext import context
-from fresco.routeargs import RouteKwarg
+from fresco.routeargs import RouteArg
 from fresco.util.urls import join_path
 
 RFC2518_METHODS = \
@@ -188,8 +188,7 @@ class AnyConverter(StrConverter):
 
         "/<lang:any('fr', 'en', 'de')>"
     """
-
-    pattern = r'[+-]?\d+'
+    pattern = None
 
     def __init__(self, *args):
         super(AnyConverter, self).__init__(None)
@@ -503,18 +502,18 @@ class Route(object):
     #: The default class to use for URL pattern matching
     pattern_class = ExtensiblePattern
 
-    def __init__(self, pattern, methods, view, view_kwargs=None,
-                 view_args=None, name=None, predicate=None, decorators=None,
-                 **kwargs):
+    def __init__(self, pattern, methods, view, kwargs=None,
+                 args=None, name=None, predicate=None, decorators=None,
+                 **_kwargs):
 
         """
         :param pattern:     A string that can be compiled into a path pattern
         :param methods:     The list of HTTP methods the view is bound to
                             ('GET', 'POST', etc)
         :param view:        The view function.
-        :param view_kwargs: A dictionary of default keyword arguments to pass
+        :param kwargs:      A dictionary of default keyword arguments to pass
                             to the view callable
-        :param view_args:   Positional arguments to pass to the view callable
+        :param args:        Positional arguments to pass to the view callable
         :param name:        A name that can later be used to retrieve the route
                             for URL generation
         :param predicate:   A callable that is used to decide whether to match
@@ -522,7 +521,7 @@ class Route(object):
                             object as its only parameter and return a boolean.
         :param decorators:  Decorator functions to apply to the view callable
                             before invoking it
-        :param kwargs:      Other keyword aruments passed through to the view
+        :param **_kwargs:   Other keyword aruments passed through to the view
                             callable.
 
 
@@ -600,14 +599,20 @@ class Route(object):
 
         p_args, p_kwargs = pattern.path_argument_info()
         for k in p_kwargs:
-            default = kwargs.pop(k + '_default', _marker)
+            default = _kwargs.pop(k + '_default', _marker)
             if default is not _marker:
                 self.routed_args_default[k] = default
 
-        self.view_args = tuple(view_args or tuple())
-        self.view_kwargs = dict(view_kwargs or {}, **kwargs)
+        self.view_args = tuple(args or _kwargs.pop('view_args', tuple()))
+        self.view_kwargs = dict(kwargs or _kwargs.pop('view_kwargs', {}),
+                                **_kwargs)
+
+        for arg in self.view_args:
+            if isinstance(arg, RouteArg):
+                arg.configure(self, None)
+
         for argname, arg in self.view_kwargs.items():
-            if isinstance(arg, RouteKwarg):
+            if isinstance(arg, RouteArg):
                 arg.configure(self, argname)
 
         self.viewspec = view
@@ -671,9 +676,10 @@ class Route(object):
         """\
         Return copy of the route bound to a given instance.
 
-        Use this when traversing view classes
+        Use this when traversing view classes.
         """
-        copy = Route.__new__(self.__class__)
+        cls = self.__class__
+        copy = cls.__new__(cls)
         copy.__dict__ = self.__dict__.copy()
         copy._setinstance(instance)
         return copy
@@ -754,9 +760,12 @@ class RouteCollection(object):
     - Can map from a request to a view
     """
 
-    def __init__(self, routes=None):
+    route_class = Route
+
+    def __init__(self, routes=None, route_class=None):
         self.__routes__ = []
         self.__routed_views__ = {}
+        self.route_class = route_class or self.route_class
         if routes is not None:
             for item in routes:
                 self.add_route(item)
@@ -869,7 +878,7 @@ class RouteCollection(object):
                 self.__routed_views__[viewspec] = route
                 return route
 
-    def get_routes(self, path, method):
+    def get_routes(self, path, method, request=None):
         """
         Generate routes matching the given path and method::
 
@@ -878,12 +887,12 @@ class RouteCollection(object):
                 print("Arguments extracted from the path:", rt.args, rt.kwargs)
                 print("RouteCollections are:", rt.collections_traversed)
 
-        :param request: a :class:`~fresco.request.Request` object
         :param path: the URL path to match (usually this is ``PATH_INFO``)
         :param method: the HTTP method to match (usually this is
                        ``REQUEST_METHOD``). May be ``None``, in which case
                        matching will be performed on the ``path`` argument
                        only.
+        :param request: a :class:`~fresco.request.Request` object
 
         :return: A generator yielding ``RouteTraversal`` objects
         """
@@ -893,12 +902,24 @@ class RouteCollection(object):
             if result is None:
                 continue
 
-            view_args = result.args
+            if request and route.predicate and not route.predicate(request):
+                continue
+
+            view_args = route.view_args + result.args
             view_kwargs = {}
             if route.view_kwargs:
                 view_kwargs.update(route.view_kwargs)
             view_kwargs.update(result.kwargs)
 
+            if request:
+                view_args = tuple(
+                    (item(request) if isinstance(item, RouteArg) else item)
+                    for item in view_args)
+
+                for k, v in view_kwargs.items():
+                    if isinstance(v, RouteArg):
+                        view_kwargs[k] = v(request)
+
             if isinstance(route, DelegateRoute):
                 try:
                     sub_routes = route.routecollectionfactory(*view_args,
@@ -907,7 +928,8 @@ class RouteCollection(object):
                 except ResponseException as e:
                     def raiser(*args, **kwargs):
                         raise e
-                    yield RouteTraversal(Route('/', ALL_METHODS, raiser),
+                    yield RouteTraversal(self.route_class('/', ALL_METHODS,
+                                                          raiser),
                                          (), {},
                                          [(self, '')])
                     continue
@@ -917,7 +939,8 @@ class RouteCollection(object):
                     view_args = ()
                     view_kwargs = {}
                 for sub in sub_routes.get_routes(result.path_remaining,
-                                                 method):
+                                                 method,
+                                                 request):
                     traversed = [(self, '')]
                     traversed.extend((c, result.path_matched + p)
                                      for c, p in sub.collections_traversed)
@@ -945,11 +968,12 @@ class RouteCollection(object):
         # Called as a decorator?
         if view is None:
             def route_decorator(func):
-                self.add_route(Route(pattern, methods, func, *args, **kwargs))
+                self.add_route(self.route_class(pattern, methods, func, *args,
+                                                **kwargs))
                 return func
             return route_decorator
 
-        route = Route(pattern, methods, view, *args, **kwargs)
+        route = self.route_class(pattern, methods, view, *args, **kwargs)
         self.add_route(route)
         return route
 
