@@ -1,10 +1,10 @@
 # coding=UTF-8
 
 from io import BytesIO
-from nose.tools import assert_equal
+from nose.tools import assert_equal, assert_raises
 
-from fresco import FrescoApp
-from fresco.compat import PY3, quote
+from fresco import FrescoApp, exceptions
+from fresco.tests import fixtures
 
 context = FrescoApp().requestcontext
 
@@ -63,48 +63,46 @@ class TestRequestProperties(object):
 
 class TestPathEncoding(object):
 
-    if PY3:
-        unquoted_url = '/el niño'
-        quoted_url = quote(unquoted_url)
-        wsgi_url = unquoted_url.encode('utf8').decode('latin1')
-    else:
-        unquoted_url = unicode('/el niño', 'UTF-8')
-        quoted_url = quote(unquoted_url.encode('UTF-8'))
-        wsgi_url = unquoted_url.encode('UTF-8')
-
-    def test_SCRIPT_NAME_PATH_INFO_latin1_decoded(self):
-
-        with context(SCRIPT_NAME=self.unquoted_url,
-                     PATH_INFO=self.unquoted_url) as c:
-            assert c.request.environ['SCRIPT_NAME'] == self.wsgi_url
-            assert c.request.environ['PATH_INFO'] == self.wsgi_url
-
     def test_url_is_quoted(self):
 
-        with context(SCRIPT_NAME=self.unquoted_url, PATH_INFO='') as c:
-            assert c.request.url == 'http://localhost' + self.quoted_url
+        with context(SCRIPT_NAME=fixtures.wsgi_unicode_path,
+                     PATH_INFO='') as c:
+            assert c.request.url == 'http://localhost' + \
+                                        fixtures.quoted_unicode_path
 
     def test_application_url_is_quoted(self):
 
-        with context(SCRIPT_NAME=self.unquoted_url, PATH_INFO='') as c:
+        with context(SCRIPT_NAME=fixtures.wsgi_unicode_path,
+                     PATH_INFO='') as c:
             assert c.request.application_url == 'http://localhost' + \
-                                                    self.quoted_url
+                                                fixtures.quoted_unicode_path
 
     def test_parsed_url_is_quoted(self):
-        with context(SCRIPT_NAME=self.unquoted_url, PATH_INFO='') as c:
-            assert c.request.parsed_url.path == self.quoted_url
+        with context(SCRIPT_NAME=fixtures.wsgi_unicode_path,
+                     PATH_INFO='') as c:
+            assert c.request.parsed_url.path == fixtures.quoted_unicode_path
 
     def test_path_is_unquoted(self):
-        with context(SCRIPT_NAME=self.unquoted_url, PATH_INFO='') as c:
-            assert c.request.path == self.unquoted_url
+        with context(SCRIPT_NAME=fixtures.wsgi_unicode_path,
+                     PATH_INFO='') as c:
+            assert c.request.path == fixtures.unquoted_unicode_path
 
     def test_script_name_is_unquoted(self):
-        with context(SCRIPT_NAME=self.unquoted_url, PATH_INFO='') as c:
-            assert c.request.script_name == self.unquoted_url
+        with context(SCRIPT_NAME=fixtures.wsgi_unicode_path,
+                     PATH_INFO='') as c:
+            assert c.request.script_name == fixtures.unquoted_unicode_path
 
     def test_path_info_is_unquoted(self):
-        with context(SCRIPT_NAME='', PATH_INFO=self.unquoted_url) as c:
-            assert c.request.path_info == self.unquoted_url
+        with context(PATH_INFO=fixtures.wsgi_unicode_path) as c:
+            assert c.request.path_info == fixtures.unquoted_unicode_path
+
+    def test_invalid_path_info_encoding_raises_bad_request(self):
+        with context(PATH_INFO=fixtures.misquoted_wsgi_unicode_path) as c:
+            assert_raises(exceptions.BadRequest, lambda: c.request.path_info)
+
+    def test_invalid_script_name_encoding_raises_bad_request(self):
+        with context(SCRIPT_NAME=fixtures.misquoted_wsgi_unicode_path) as c:
+            assert_raises(exceptions.BadRequest, lambda: c.request.script_name)
 
 
 class TestMakeURL(object):
@@ -172,6 +170,31 @@ class TestMakeURL(object):
             assert_equal(c.request.make_url(path='/a b'),
                          'http://localhost/a%20b')
 
+    def test_replace_query_replaces_existing(self):
+        with context(QUERY_STRING='a=1') as c:
+            assert_equal(c.request.make_url(query_replace={'a': '2'}),
+                         'http://localhost/?a=2')
+
+    def test_replace_query_adds_new_value(self):
+        with context(QUERY_STRING='a=1') as c:
+            assert_equal(c.request.make_url(query_replace={'b': '2'}),
+                         'http://localhost/?a=1;b=2')
+
+    def test_add_query_doesnt_replace_existing(self):
+        with context(QUERY_STRING='a=1') as c:
+            assert_equal(c.request.make_url(query_add={'a': '2'}),
+                         'http://localhost/?a=1;a=2')
+
+    def test_add_query_works_on_specified_query(self):
+        with context(QUERY_STRING='a=1') as c:
+            assert_equal(c.request.make_url(query='', query_add={'a': '2'}),
+                         'http://localhost/?a=2')
+
+    def test_add_query_does_not_mutate_request_query(self):
+        with context(QUERY_STRING='a=1') as c:
+            c.request.make_url(query_add={'a': '2'})
+            assert list(c.request.query.allitems()) == [('a', '1')]
+
 
 class TestResolveURL(object):
 
@@ -234,7 +257,7 @@ class TestFiles(object):
 
     request_args = {'wsgi_input': post_data,
                     'REQUEST_METHOD': 'POST',
-                    'CONTENT_TYPE': 'multipart/form-data; boundary=' + \
+                    'CONTENT_TYPE': 'multipart/form-data; boundary=' +
                                         boundary,
                     'CONTENT_LENGTH': str(len(post_data))}
 
