@@ -67,12 +67,11 @@ class TestRequestContext(object):
             raise AssertionError("Output does not appear interleaved")
         assert_equal(len(output), threadcount * itercount)
 
-
-    def test_context_inherits_from_parent(self):
+    def test_context_does_not_inherit_from_parent(self):
         c = RequestContext()
         c.push(foo=1)
         c.push(bar=2)
-        assert_equal(c.foo, 1)
+        assert_raises(AttributeError, lambda: c.foo)
 
     def test_child_context_overrides_parent(self):
         c = RequestContext()
@@ -87,4 +86,3 @@ class TestRequestContext(object):
         assert_equal(c.foo, 2)
         c.pop()
         assert_equal(c.foo, 1)
-
